import Model

extension Board: Codable {
    enum CodingKeys: CodingKey {
        case width, height, content
    }
    
    public init(from decoder: any Decoder) throws {
        let container = try decoder.container(keyedBy: CodingKeys.self)
        
        let width = try container.decode(Int.self, forKey: .width)
        let height = try container.decode(Int.self, forKey: .height)
        let pieces = try container.decode([Piece?].self, forKey: .content)
        
        if pieces.count != width * height {
            throw DeError.InvalidData
        }
        
        guard var zelf = Self.init(columns: width, rows: height) else {
            throw DeError.InitializationFailed
        }
        
        zelf.coords.forEach { coord in
            zelf[coord] = pieces[coord.toIndex(width: width)]
        }
        
        self = zelf
    }
    
    public func encode(to encoder: any Encoder) throws {
        var container = encoder.container(keyedBy: CodingKeys.self)
        
        try container.encode(self.columns, forKey: .width)
        try container.encode(self.rows, forKey: .height)
        // Array is flattened because Board inner cell data structure is private and only accessible via a sequence
        try container.encode(Array(self.cells), forKey: .content)
    }
}

extension Coords: Codable {
    public init(from decoder: any Decoder) throws {
        var container = try decoder.unkeyedContainer()
        
        self.init(try container.decode(Int.self), try container.decode(Int.self))
    }
    
    public func encode(to encoder: any Encoder) throws {
        var container = encoder.unkeyedContainer()
        
        try container.encode(self.col)
        try container.encode(self.row)
    }
}
